# -*- coding: utf-8 -*-

import redis
import json
from ..base_consumer import CocoBaseConsumer

class CocoRedisConsumer(CocoBaseConsumer):

    def __init__(self, conf, worker, logger = None):
        self._client = None
        self._host = conf['SERVER']
        self._port = int(conf['PORT'])
        self._db = conf.get("DB", 7)
        self._queue = conf.get("QUEUE")
        self._password = conf.get("PASSWORD", None)
        super().__init__(conf, worker, logger)


    def connect(self):
        self._client = redis.Redis(host = self._host, 
                                   port = self._port, 
                                   decode_responses = True, 
                                   db = self._db,
                                   password = self._password)
        self._client.ping()

        while True:
          value = self._client.blpop(self._queue, 0)[1]
          self._worker.process(value)



import pika

def createBlockingConnection(config):
	credentials = pika.PlainCredentials(config['USERNAME'], config['PASSWORD'])
	parameters = pika.ConnectionParameters(config['SERVER_ADDRESS'],
	                                   config['SERVER_PORT'],
	                                   config.get("VIRTUAL_HOST", "/"),
	                                   credentials, 
	                                   heartbeat=config.get("HEARTBEAT", 0))
	connection = pika.BlockingConnection(parameters)
	channel = connection.channel()

	channel.exchange_declare(exchange=config['EXCHANGE_NAME'],
	                     exchange_type=config['EXCHANGE_TYPE'])
	channel.queue_declare(queue=config['QUEUE_NAME'], durable=True)

	return connection, channel



from multiprocessing import Pool
from .rabbitmq.rmq_consumer import CocoRMQConsumer
from .kafka.kafka_consumer import CocoKafkaConsumer
from .redis.redis_consumer import CocoRedisConsumer
import logging
logger = logging.getLogger(__name__)

# this is aka mixture of a consumer factory and threadpool
class CocoConsumerManager(object):

    CONSUMER_CLASS = {
        "RMQ": CocoRMQConsumer,
        "KAFKA": CocoKafkaConsumer,
        "REDIS": CocoRedisConsumer
    }


    def __init__(self, config, worker_class, pool_size, customized_logger = None):
        self._worker_class = worker_class
        self._pool = None
        self._pool_size = pool_size
        self._config = config
        if customized_logger:
            logger = customized_logger


    def start(self):
        self._pool = Pool()
        results = [self._pool.apply_async(CocoConsumerManager._start_consumer, args=[x, self._worker_class, self._config]) for x in range(self._pool_size)]
        self._pool.close()
        self._pool.join()


    @staticmethod
    def _start_consumer(seq, worker_class, config):
        try:
            logger.info('start consumer')
            worker = worker_class(config, seq)

            consumer_type = config["MQ_TYPE"]
            logger.info('cusumer type: {}'.format(consumer_type))
            consumer_class = CocoConsumerManager.CONSUMER_CLASS[consumer_type]

            sub_config = config[consumer_type]
            consumer = consumer_class(sub_config, worker, logger)
            consumer.connect()

        except Exception as error:
            logger.error(error)





from .rabbitmq.rmq_producer import CocoRMQProducer
from .kafka.kafka_producer import CocoKafkaProducer
from .redis.redis_producer import CocoRedisProducer
import logging
default_logger = logging.getLogger(__name__)

class CocoProducerManager(object):

    CONSUMER_CLASS = {
        "RMQ": CocoRMQProducer,
        "KAFKA": CocoKafkaProducer,
        "REDIS": CocoRedisProducer
    }


    @staticmethod
    def create_instance(config, logger = default_logger):
        producer = None
        try:
            producer_type = config["MQ_TYPE"]
            producer_class = CocoProducerManager.CONSUMER_CLASS[producer_type]

            sub_config = config[producer_type]
            producer = producer_class(sub_config, logger)
        except Exception as err:
            logger.error(err)

        return producer



# -*- coding: utf-8 -*-

import redis
import json
import pika
from ..base_producer import CocoBaseProducer
from .common import createBlockingConnection


class CocoRMQProducer(CocoBaseProducer):

    def __init__(self, conf, logger = None):
        self._exchange_name = conf['EXCHANGE_NAME']
        self._queue_name = conf['QUEUE_NAME']
        self._connection = None
        self._channel = None
        super().__init__(conf, logger)


    def connect(self):
        self._connection, self._channel = createBlockingConnection(self._config)


    def send(self, data):
      self._channel.basic_publish(exchange=self._exchange_name, 
                      routing_key=self._queue_name, 
                      body=data, 
                      properties=pika.BasicProperties(
                         delivery_mode = 2, # make message persistent
                      ))


    def close(self):
        self._connection.close()





from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))
# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
	name='cocotask',
    
	version='0.1.1.10',
    
    description='An unified framework to build task queue on Rabbitmq | Kafka | Redis',
    long_description=long_description,
    long_description_content_type='text/markdown', 

    url='https://github.com/likexx/cocotask',
    
    author='Like Zhang',
    
    author_email='zlike@yahoo.com',

    license='MIT',  
    
#    packages=[''],
    packages=find_packages(exclude=['release.sh']),

    zip_safe=False,
      
    entry_points={
        'console_scripts': [
            'cocotask = cocotask.taskrunner:main',
        ],
    },

	keywords='task message queue MQ Pika kafka-python rabbitmq kafka redis rmq producer consumer celery',
    
    install_requires=['pika', 'kafka-python', 'redis', 'jsmin'],

    project_urls={  # Optional
        'Bug Reports': 'https://github.com/likexx/cocotask/issues',
        'Funding': 'https://github.com/likexx/cocotask/',
        'Say Thanks!': 'https://github.com/likexx/cocotask/',
        'Source': 'https://github.com/likexx/cocotask/',
    },
)




# -*- coding: utf-8 -*-
import pika.exceptions as exceptions
from ..base_consumer import CocoBaseConsumer
from .common import createBlockingConnection


class CocoRMQConsumer(CocoBaseConsumer):

    def __init__(self, conf, worker, logger = None):
        self._exchange_name = conf['EXCHANGE_NAME']
        self._queue_name = conf['QUEUE_NAME']
        self._connection = None
        super().__init__(conf, worker, logger)

    def connect(self):
        self._connection, channel = createBlockingConnection(self._config)
        channel.queue_bind(exchange = self._exchange_name,
                           queue = self._queue_name)

        channel.basic_qos(prefetch_count=1)
        channel.basic_consume(self.on_message,
                              queue=self._queue_name)
        channel.start_consuming()

    def on_message(self, ch, method, properties, body):
        self._worker.process(body)
        try:
            ch.basic_ack(delivery_tag = method.delivery_tag)
        except exceptions.ChannelClosed as e:
            self._logger.error("channel closed. exp: %s" % repr(e))
